/**
 * Name:Daniel Greco
 * Filename: RochambeauProtocol.java
 * Description: This file will handle the rules and interaction between client
 * and host for the rochambeau (Rock-Paper-Scissors) game.
 */ 
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Scanner;

/**
 *
 * @author Administrator
 */
public class RochambeauProtocol implements Protocol{

    //state codes
    private static final int WAITING = 0;
    private static final int PLAYING = 1;
    private static final int PLAYAGAIN = 2;
    
    
    private int state;
    private String name; //host name
    private BufferedReader stdIn; //read from stdin
    private Scanner scan;
    private int wins, losses;  //this will be used so we can show a scoreboard 
                               //to the players.
    private String choice; //rock paper or scissors

    //we'll never get here
    private RochambeauProtocol()
    {

    }
    
    /**
     * Constructor for class RochambeauProtocol
     * @param name is a string that is the name of the host
     * 
     */
    public RochambeauProtocol(String name)
    {   

         //initialize the std in reader
        stdIn = new BufferedReader(new InputStreamReader(System.in));
        //set the state to the default state
        state = WAITING;
        //set up the scanner
        scan = new Scanner(System.in);
        //save the name
        this.name = name;
        //make sure to set the wins and losses to zero
        this.wins = 0;
        this.losses = 0;
        //no choice yet
        this.choice = null;
    }
  
    
    
    /**
     * processInput is the main method of this class. It'll handle the input the client sent and generate the output to give back to them.
     * @param theInput is a message object that contains the message the user gave
     * @return the host message back to the client
     */
    public Message processInput(Message theInput) throws IOException
    {
        String theOutput = null;
        //Most often it'll always be a stringmsg. However now message
        //supports guess message codes, which we'll use to allow the client
        //to validate their guess before sending it to the host, to avoid
        //any problems.
        int theMsg = Message.STRINGMSG; 
        String clientName; 
    
        //see if we have a clientName this time. If we do store it.
        try
        {
            clientName = theInput.getName();
        }
        catch(Exception e)
        {
            clientName = "the client";
        }
    
        //if we're processing input in this area, that means we're now connected
        switch(state) 
        {
            
            //default state!
            //Note: This message will not be sent until the host chooses rock paper
            //or scissor. This is one way we try to hide that the user can
            //only type back and forth with each other.
            case WAITING: 
            {
                //the only time we're in this area is when we start fresh,
                //or we are connecting to a new player. While we clear
                //the record in one other spot, this will make sure
                //that we handle in case of disconnects.
                clearRecord();
                //Step 4: Send message to client because we're now connected
                theOutput = "Connection to " + name + " established\nLet the games begin!\nEnter Rock Paper or Scissors";
                System.out.println("Player Connected");
                //change state to playing as we now have a connection
                state = PLAYING;
                //get the host's rock paper or scissor
                getChoice();
                //tells the client that this is a guess message, meaning the
                //client's next message will be a guess to. So make sure
                //they know to validate before sending
                theMsg = Message.GUESS;
                break;
            }
        
            case PLAYING:
            {
                //check if the player quitted
                if(theInput.isQuit())
                {
            //go back to waiting and sent the quit message back. So they know to leave
                    state = WAITING;
                    System.out.println(theInput);
                    theOutput = "Bye";
                    //make sure to set wins and losses to zero, so we know
                    clearRecord();
                }   
                else
                {
                    //Step 10: Print message on screen
                    System.out.println(clientName +  " threw down " + theInput);
                    //Step 11:Read Message from user
                    if(theInput.getMsgType() == Message.GUESS)
                    {   
                        String message = theInput.getMessage();
                        //remove white space. Not important but keeps formatting 
                        //in check.
                        message = message.replace(" ", "");
        
                        //start to build the normal string to send back
                        String guessOutput = compareChoices(message);
                        //if they got to this then they won
                        if(Message.isOver(guessOutput))
                        {
                            //inform the client the game ended.
                            theOutput = this.name + " threw down " + this.choice + "\n";
                            theOutput += guessOutput;
                            //displays the client's record. I'm flipping losses and wins, since this
                            //is the host's record reversed.
                            theOutput+= "\nYour current record against " + this.name + " is " + this.losses + "-" + this.wins;
                            //make sure to set the state to playagain
                            this.state = PLAYAGAIN;
                            //Give the host their win loss record
                            System.out.println("\nYour current record against " + clientName + " is " + this.wins + "-" + this.losses);
                            System.out.println("If you'd like to leave feel free to do so. The client is about to choose if we're playing another round");
                            return new Message(name, theOutput, theMsg);
                        }
                        //the players tied so get a new choice from the host
                        else if(guessOutput.equalsIgnoreCase("You tied! Time to throw down again!\n"))
                        {
                            theOutput = this.name + " threw down " + this.choice + "\n";
                            theOutput += guessOutput;
                            getChoice();
                            theMsg = Message.GUESS;
                            return new Message(name, theOutput, theMsg);
                        }
                        //add the output to the return string
                        theOutput = guessOutput; 
                    }
                }
            }
                break;
            //this will be reached when there is an outcome to the rochambeau
            //match
            case PLAYAGAIN:
            {
                //get the message
                String message = theInput.getMessage();
                    //if they don't give yes, they're quitting
                    if(message.equalsIgnoreCase("Yes"))
                    {
                        //get a new number
                        System.out.println("Here we go again (not the OK Go song but the actual meaning)");
                        state = PLAYING;
                        theOutput = "To quote coolio \"Aw Here It Goes\". Enter Rock Paper or Scissors Again!";
                        theMsg = Message.GUESS;
                        getChoice();
                    }
                    
                    else
                    {
                        //go back to waiting
                        state = WAITING;
                        System.out.println(theInput);
                        //send a quit message to the client, so they can disconnect.
                        theOutput = "Bye";
                        
                    } 
                break;
            }
            default:
                //we should never get here
                break;
        }
        //return the message if we haven't already
        return new Message(name, theOutput, theMsg);
    } //processinput
    
    /***************************************************************************
     * getChoice will get the host's choice of rock, paper or scissors. It
     * will be a loop to make sure that we get a valid choice
     * 
     **************************************************************************/
    private void getChoice()
    {
        
        //this will be true if the host gives a valid choice
        boolean goOn = false;
        
        do
        {
            System.out.println("Enter Rock, Paper, or Scissors Now!");
            System.out.print(">");
            //read in the choice
            this.choice = scan.next();
            //check if it's valid
            goOn = isValidChoice(this.choice);
            //if it's not valid display an error message.
            if(!goOn)
            {
                System.out.println("You gave an invalid entry. Remember Rock, Paper, or Scissors");
            }
        //run until we get a valid message
        }while(!goOn);
    }
    
    /**
     * isValidChoice will take in a string representing a choice the user has 
     * typed. 
     * 
     * @param message is a string that will represent the choice the user has
     * given.
     * @return returns true if it's a valid choice, false otherwise.
     */
    public static boolean isValidChoice(String message)
    {
        //case insensitive
        if(message.equalsIgnoreCase("Rock") ||
                message.equalsIgnoreCase("Scissors") ||
                message.equalsIgnoreCase("Paper"))
            return true;
        else
            return false;
            
    }
    
    /**
     * compareChoices will compare what the host and client gave, and will declare
     * who wins loses or ties.  Note both choices are assumed valid before hand.
     * Otherwise you will have problems.
     * 
     * @param client is a string representing the choice the client gave.
     * @return a string that will hold what message to give to the client.
     */
    private String compareChoices(String client)
    {
        //host and client throw same choice
        if(client.equalsIgnoreCase(this.choice))
        {
                System.out.println("You tied! Time to throw down again!");
                return "You tied! Time to throw down again!\n";
        }
        //host gives scissors, client gives rock
        else if(this.choice.equalsIgnoreCase("scissors") && client.equalsIgnoreCase("rock"))
        {
                System.out.println("You Won!");
                ++this.wins;
                return "You got destroyed!\n";
        }
        //host gives scissors, client gives paper
        else if(this.choice.equalsIgnoreCase("scissors") && client.equalsIgnoreCase("paper"))
        {
                System.out.println("You got destroyed!");
                ++this.losses;
                return "You Won!\n";
        }
        //host gives paper, client gives rock
        else if(this.choice.equalsIgnoreCase("paper") && client.equalsIgnoreCase("rock"))
        {
            System.out.println("You Won!");
            ++this.wins;
            return "You got destroyed!\n";
        }
        //host gives paper, client gives scissors
        else if(this.choice.equalsIgnoreCase("paper") && client.equalsIgnoreCase("scissors"))
        {
            System.out.println("You got destroyed!");
            ++this.losses;
            return "You Won!\n";
        }
        //host gives rock, client gives scissors
        else if(this.choice.equalsIgnoreCase("rock") && client.equalsIgnoreCase("scissors"))
        {
            System.out.println("You Won!");
            ++this.wins;
            return "You got destroyed!\n";
        }
        //host gives rock, client gives paper 
        else
        {
            System.out.println("You got destroyed!");
            ++this.losses;
            return "You Won!\n";
        }
    }
    
    /**
     * clearRecord is a very simple method that will simply set the wins and
     * losses back to zero.
     */
    private void clearRecord()
    {
        this.wins = 0;
        this.losses = 0;
    }
}
